#Step 1 - Insert Doors
#Version 1.0

require 'sketchup.rb'

module Insert_doors
 
  def Insert_doors.main

mod = Sketchup.active_model # Open model
ent = mod.entities # All entities in model
sel = mod.selection # Current selection
mod.start_operation "Insert Doors"

prompts = ['Tolerance (Inches)  ']
defaults = [3.to_l]
results = UI.inputbox prompts, defaults, 'Door Detection Tolerance (Secondary Test)'
detection_dist = results[0]

c = Sketchup::Set.new
d = Sketchup::Set.new

doortype = 'fixed'

comp_directory = ENV["USERPROFILE"] + "\\Documents\\Sketchup\\Custom Components\\"

if doortype == 'dynamic'

   #######Dynamic Door Components#######
   lh_door_path = comp_directory + 'Door_LH.skp'
   lh_door_def = Sketchup.active_model.definitions.load lh_door_path
   rh_door_path = comp_directory + 'Door_RH.skp'
   rh_door_def = Sketchup.active_model.definitions.load rh_door_path
   #######Dynamic Door Components#######

else

   #######Fixed Door Width Components - utilizes instance variables for door components#######
   (24..60).step(2) do |i|
      hand = [ ['RH_','rh_'], ['LH_','lh_'] ]
      hand.each do |j|
         door_i_path = comp_directory + ['Door_',j[0],i,'.skp'].join
         door_i_def = Sketchup.active_model.definitions.load door_i_path
         var_i_name = [j[1],'door_',i,'_def'].join
         instance_variable_set("@#{var_i_name}", door_i_def)
      end
   end 
   
   undefined_door_neither_path = comp_directory + 'Undefined_Door_Neither.skp'
   undefined_door_neither_def = Sketchup.active_model.definitions.load undefined_door_neither_path
   
   undefined_door_both_path = comp_directory + 'Undefined_Door_Both.skp'
   undefined_door_both_def = Sketchup.active_model.definitions.load undefined_door_both_path
   
   undefined_door_size_path = comp_directory + 'Undefined_Door_Size.skp'
   undefined_door_size_def = Sketchup.active_model.definitions.load undefined_door_size_path
   
   #######Fixed Door Width Components#######

end # if for dynamic/fixed

# define Curve and Arc Curve selection criteria
min_radius = 24
max_radius = 60
min_angle_degrees = 30
max_angle_degrees = 200
min_curve_length = 24

sel.each do |e| # Add Curve entities to set d
  if e.is_a?(Sketchup::Edge) && e.curve && !e.curve.is_a?(Sketchup::ArcCurve) && e.curve.length > min_curve_length && (e.curve.first_edge.start.position != e.curve.last_edge.end.position)   
    d.insert e.curve
  end  
end

d.each do |e|  # method to replace Curve entities heuristically with ArcCurve entities   
   start_segment = e.first_edge
   end_segment = e.last_edge
   start_segment_start_pt = start_segment.start.position
   start_segment_end_pt = start_segment.end.position
   end_segment_start_pt = end_segment.start.position
   end_segment_end_pt = end_segment.end.position
   orig_vec_start_segment = start_segment.line[1]
   orig_vec_end_segment = end_segment.line[1]
   start_segment_mid_point = Geom.linear_combination 0.5, start_segment_start_pt, 0.5, start_segment_end_pt
   end_segment_mid_point = Geom.linear_combination 0.5, end_segment_start_pt, 0.5, end_segment_end_pt
   
   unless start_segment_mid_point == end_segment_mid_point   
      arc_normal = orig_vec_start_segment.cross orig_vec_end_segment
      start_cross_vector = orig_vec_start_segment.cross arc_normal
      end_cross_vector = orig_vec_end_segment.cross arc_normal
        
      start_segment_line = [start_segment_mid_point, start_cross_vector]
      end_segment_line = [end_segment_mid_point, end_cross_vector]
      arc_center_pt = Geom.intersect_line_line(start_segment_line, end_segment_line)
   end
   
   arc_x_axis = Geom::Vector3d.new 1,0,0
   arc_radius = arc_center_pt.distance start_segment_start_pt 
   arc_start_vector = arc_center_pt.vector_to ( start_segment_start_pt)
   arc_end_vector = arc_center_pt.vector_to ( end_segment_end_pt)  
   arc_x_axis = arc_start_vector 
   arc_start_angle = arc_start_vector.angle_between arc_x_axis
   arc_end_angle = arc_end_vector.angle_between arc_x_axis
   
   entity = ent.add_arc arc_center_pt, arc_x_axis, arc_normal, arc_radius, arc_start_angle, arc_end_angle
   sel.add entity
   
   temp_array = []
   temp_array << e.edges
   
   temp_array.each do |f|
      ent.erase_entities f
   end

end

sel.each do |e| # Add ArcCurve entities to Set c
  if e.is_a?(Sketchup::Edge) && e.curve.is_a?(Sketchup::ArcCurve) && e.curve.radius > min_radius && e.curve.radius < max_radius && (e.curve.end_angle - e.curve.start_angle).radians < max_angle_degrees && (e.curve.end_angle - e.curve.start_angle).radians > min_angle_degrees
     c.insert e.curve
  end
end

location_ary = []
new_doors_placed = 0
undefined_doors_placed = 0
undefined_doors_replaced = 0
c.each do |e|
   location = e.center   
   unless location_ary.include? location
      location_ary << location
      doorsize = e.radius

      startpt = e.first_edge.start.position
      endpt = e.last_edge.end.position

      tempvectorA = location.vector_to( startpt ) # Create temporary vectors to start and end points of arc
      tempvectorB = location.vector_to( endpt )  
    
      #primary test of A vector
      testvector_A = tempvectorA # Create a test vector and point beyond the endpoint of testvectorA
      testvector_A.length = tempvectorA.length - 3

      testpoint_A = location + testvector_A
      testpoint_A.z = location.z + 1

      ray_A = testpoint_A, Geom::Vector3d.new(0, 0, -1)
      result_A = mod.raytest(ray_A, false)
 
      #secondary test of A vector
      testpoint_A_1 = location + testvector_A
      if result_A.nil?
         testvector_A_2 = testvector_A.cross Geom::Vector3d.new(0, 0, -1)
         testvector_A_3 = testvector_A.cross Geom::Vector3d.new(0, 0, 1)
         
         ray_A_2 = testpoint_A_1, testvector_A_2
         ray_A_3 = testpoint_A_1, testvector_A_3

         result_2_A = mod.raytest(ray_A_2, false)
         result_3_A = mod.raytest(ray_A_3, false)
                  
         if !result_2_A.nil?
            distance_2_A = result_2_A[0].distance testpoint_A
            if distance_2_A < detection_dist
               result_A = result_2_A
            end
         end 
         
         if !result_3_A.nil?
            distance_3_A = result_3_A[0].distance testpoint_A
            if distance_3_A < detection_dist
               result_A = result_3_A
            end
         end
      end # secondary test of A vector   
      
      #primary test of B vector
	  testvector_B = tempvectorB # Create a test vector and point beyond the endpoint of testvectorA
      testvector_B.length = tempvectorB.length - 3

      testpoint_B = location + testvector_B
      testpoint_B.z = location.z + 1

      ray_B = testpoint_B, Geom::Vector3d.new(0, 0, -1)
      result_B = mod.raytest(ray_B, false)
 
      #secondary test of B vector
      testpoint_B_1 = location + testvector_B
      if result_B.nil?
         testvector_B_2 = testvector_B.cross Geom::Vector3d.new(0, 0, -1)
         testvector_B_3 = testvector_B.cross Geom::Vector3d.new(0, 0, 1)
         
         ray_B_2 = testpoint_B_1, testvector_B_2
         ray_B_3 = testpoint_B_1, testvector_B_3

         result_2_B = mod.raytest(ray_B_2, false)
         result_3_B = mod.raytest(ray_B_3, false)
                  
         if !result_2_B.nil?
            distance_2_B = result_2_B[0].distance testpoint_B
            if distance_2_B < detection_dist
               result_B = result_2_B
            end
         end 
         
         if !result_3_B.nil?
            distance_3_B = result_3_B[0].distance testpoint_B
            if distance_3_B < detection_dist
               result_B = result_3_B
            end
         end
      end # secondary test of A vector

      if result_A.nil? && !result_B.nil? # Set the vectors for the door open and closed position based on the test point location
         vectorDoorOpen = tempvectorB
         vectorDoorClosed = tempvectorA
      elsif result_B.nil? && !result_A.nil?
         vectorDoorOpen = tempvectorA
         vectorDoorClosed = tempvectorB
	  elsif result_A.nil? && result_B.nil?
	     vectorDoorOpen = tempvectorB
         vectorDoorClosed = tempvectorA
		 doorsize = 1000
      elsif !result_A.nil? && !result_B.nil?
	     vectorDoorOpen = tempvectorB
         vectorDoorClosed = tempvectorA
		 doorsize = 2000
      end

      if vectorDoorClosed.y < 0 # Set angle in radians from 0 to 180, and 180 and 360 degrees
         angle = -(vectorDoorClosed.angle_between [1,0,0])
      else
         angle = (vectorDoorClosed.angle_between [1,0,0])
      end
   
      if angle.radians.abs > 90 # test whether door angle is within an angle tolerance for alignment purposes
         testangle = angle.radians.abs - 90
      else
         testangle = angle.radians.abs
      end  
      angle_tolerance = 10
      if (testangle) < angle_tolerance || (90 - testangle) < angle_tolerance
         angle = (angle/ 90.degrees).round(0) * 90.degrees
      end
          
      rotation = Geom::Transformation.rotation [0,0,0], [0,0,1], angle
      translation = Geom::Transformation.new location

      vectorproduct = vectorDoorOpen.cross vectorDoorClosed # Compute cross product of vectors to determine relative position

      if doortype == 'dynamic'   
      #######Dynamic Door Components####### 
         if vectorproduct.z > 0 # Place a LH or RH door based on the vectors relative position.
            door_def = rh_door_def
         elsif vectorproduct.z < 0
            door_def = lh_door_def
         end
         door_def.set_attribute 'dynamic_attributes', 'doorwidth', doorsize
      #######Dynamic Door Components#######

      else
      #######Fixed Door Width Components#######
         if vectorproduct.z > 0 # Place a LH or RH door based on the vectors relative position.
            case doorsize
			   when 23...25
	              door_def = instance_variable_get(:@rh_door_24_def)
			   when 25...27
	              door_def = instance_variable_get(:@rh_door_26_def)
			   when 27...29
	              door_def = instance_variable_get(:@rh_door_28_def)
	           when 29...31
		          door_def = instance_variable_get(:@rh_door_30_def)
	           when 31...33
		          door_def = instance_variable_get(:@rh_door_32_def)
	           when 33...35
		          door_def = instance_variable_get(:@rh_door_34_def)
	           when 35...37
		          door_def = instance_variable_get(:@rh_door_36_def)
			   when 37...39
		          door_def = instance_variable_get(:@rh_door_38_def)
			   when 39...41
		          door_def = instance_variable_get(:@rh_door_40_def)
	           when 41...43
		          door_def = instance_variable_get(:@rh_door_42_def)
			   when 43...45
		          door_def = instance_variable_get(:@rh_door_44_def)
			   when 45...47
		          door_def = instance_variable_get(:@rh_door_46_def)
			   when 47...49
		          door_def = instance_variable_get(:@rh_door_48_def)
			   when 49...51
		          door_def = instance_variable_get(:@rh_door_50_def)
			   when 51...53
		          door_def = instance_variable_get(:@rh_door_52_def)
			   when 53...55
		          door_def = instance_variable_get(:@rh_door_54_def)
			   when 55...57
		          door_def = instance_variable_get(:@rh_door_56_def)
			   when 57...59
		          door_def = instance_variable_get(:@rh_door_58_def)
			   when 59...61
		          door_def = instance_variable_get(:@rh_door_60_def)
			   when 1000
		          door_def = undefined_door_neither_def
               when 2000
		          door_def = undefined_door_both_def
	           else
	              door_def = undefined_door_size_def
           end
         elsif vectorproduct.z < 0
            case doorsize
			   when 23...25
	              door_def = instance_variable_get(:@lh_door_24_def)
			   when 25...27
	              door_def = instance_variable_get(:@lh_door_26_def)
			   when 27...29
	              door_def = instance_variable_get(:@lh_door_28_def)
	           when 29...31
		          door_def = instance_variable_get(:@lh_door_30_def)
	           when 31...33
		          door_def = instance_variable_get(:@lh_door_32_def)
	           when 33...35
		          door_def = instance_variable_get(:@lh_door_34_def)
	           when 35...37
		          door_def = instance_variable_get(:@lh_door_36_def)
			   when 37...39
		          door_def = instance_variable_get(:@lh_door_38_def)
			   when 39...41
		          door_def = instance_variable_get(:@lh_door_40_def)
	           when 41...43
		          door_def = instance_variable_get(:@lh_door_42_def)
			   when 43...45
		          door_def = instance_variable_get(:@lh_door_44_def)
			   when 45...47
		          door_def = instance_variable_get(:@lh_door_46_def)
			   when 47...49
		          door_def = instance_variable_get(:@lh_door_48_def)
			   when 49...51
		          door_def = instance_variable_get(:@lh_door_50_def)
			   when 51...53
		          door_def = instance_variable_get(:@lh_door_52_def)
			   when 53...55
		          door_def = instance_variable_get(:@lh_door_54_def)
			   when 55...57
		          door_def = instance_variable_get(:@lh_door_56_def)
			   when 57...59
		          door_def = instance_variable_get(:@lh_door_58_def)
			   when 59...61
		          door_def = instance_variable_get(:@lh_door_60_def)
			   when 1000
		          door_def = undefined_door_neither_def
               when 2000
		          door_def = undefined_door_both_def
	           else
	              door_def = undefined_door_size_def
           end
         end
      #######Fixed Door Width Components#######
      end # if for dynamic/fixed
      
      comp_name = door_def.name

      #test whether a door component already exists at the location
	  comp_testpoint = location
	  comp_testpoint.z = location.z + 1000
	  ray_comp = comp_testpoint, Geom::Vector3d.new(0,0,-1)
	  result_comp = mod.raytest(ray_comp,false)
      comp_testpoint.z = comp_testpoint.z - 1000
          
      if result_comp && result_comp[1][0].is_a?(Sketchup::ComponentInstance)
	     comp_found_name = result_comp[1][0].definition.name
      end
      
      if comp_found_name.nil?
         instance = ent.add_instance door_def, translation*rotation
         
         if ( comp_name.start_with?('Door_RH') || comp_name.start_with?('Door_LH') )
            new_doors_placed += 1
         elsif comp_name.start_with?('U')
            undefined_doors_placed += 1
         end
         
      elsif comp_found_name && comp_found_name.start_with?('U') && ( comp_name.start_with?('Door_RH') || comp_name.start_with?('Door_LH') )
         ent.erase_entities result_comp[1][0]
         instance = ent.add_instance door_def, translation*rotation
         
         if ( comp_name.start_with?('Door_RH') || comp_name.start_with?('Door_LH') )
            undefined_doors_replaced += 1
         elsif comp_name.start_with?('U')
            undefined_doors_placed += 1
         end
         
      end

      if doortype == 'dynamic'
      #######Dynamic Door Components#######
         dcs = $dc_observers.get_latest_class # Update component dynamic attributes
         dcs.redraw_with_undo(instance)
      #######Dynamic Door Components#######
      end # if for dynamic/fixed
      
   end # unless arc/curve location already detected

end # do

text = "#{new_doors_placed} new door(s) placed" + "\n" +
       "#{undefined_doors_placed} undefined door(s) placed" + "\n" +
       "#{undefined_doors_replaced} undefined door(s) replaced"
UI.messagebox(text, MB_OK)

mod.commit_operation

	end # .main method
 
end # Insert_doors module

#----------------menu------------------
if !file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Step 1 - Insert Doors") { Insert_doors.main }
end
 
file_loaded(__FILE__)
