#Step 2 - Connect Components With Edges
#Version 1.2

require 'sketchup.rb'

module Connect_components_with_edges
 
   def Connect_components_with_edges.main

start = Time.now

mod = Sketchup.active_model # Open model
ent = mod.entities # All entities in model
sel = mod.selection # Current selection
mod.start_operation "Connect Components With Edges"

hallpath = []
allcomps = []
nodes = []
doors = []
sel_comp = []
edge_startpts = []
edge_endpts = []
 
#input box to identify component type to connect
prompts = ['Component type to connect: ']
defaults = ["Doors"]
list = ["Doors|Nodes"]
input = UI.inputbox prompts, defaults, list, 'Connect Components With Edges'
dd_sel = input[0]

#select all door and node components from selection
sel.each do |e|
   allcomps << e if e.is_a?(Sketchup::ComponentInstance) 
end
nodes = allcomps.select {|e| e.definition.name.start_with?('Room', 'Stair', 'Elevator') }
doors = allcomps.select {|e| e.definition.name.start_with?('Door')}

nodes = nodes.sort_by{|e| e.bounds.center.x}
doors = doors.sort_by{|e| e.bounds.center.x}

#shift all comps upward
allcomps.each do |e|
   v = Z_AXIS.clone
   if e.definition.name.include?("Profile")
      v.length = 2
   else
      v.length = 1
   end
   e.transform!(v)
end

if dd_sel == "Doors"
   sel_comps = doors
else
   sel_comps = nodes
end

#select all hallpaths in the current slection
sel.each do |e|
   hallpath << e if e.is_a? Sketchup::Edge
end

hallpath = hallpath.sort_by{|e| e.bounds.center.x}

#find the shortest direct path from each node to every hallpath 'line'.  If the point is not on the hallpath itself, draw line to closest vertex
sel_comps.each do |sel_comp|
   testdist = []
   cand_edge_startpts = []
   cand_edge_endpts = []
   comp_centerpt = sel_comp.bounds.center
   comp_centerpt.z = hallpath[0].bounds.corner(0).z
  
   loop_counter = 0
   current_loop_index = 0
  
   hallpath.each do |segment|

      loop_counter = loop_counter + 1

      hallpath_segment = segment.line
      projected_point = comp_centerpt.project_to_line hallpath_segment
      dist_to_hallpath = comp_centerpt.distance projected_point
      
      if dist_to_hallpath == 0
         edge_startpts << projected_point
         edge_endpts << projected_point
      end
      
      if dist_to_hallpath < 25*12 && dist_to_hallpath != 0
         startpt = segment.start
         endpt = segment.end
      
         if segment.bounds.contains? projected_point
            projected_vector = comp_centerpt.vector_to projected_point
         else #identify closest vertex and create line to it
            startdist = comp_centerpt.distance startpt
            enddist = comp_centerpt.distance endpt
            if startdist < enddist
               projected_vector = comp_centerpt.vector_to startpt
            else
               projected_vector = comp_centerpt.vector_to endpt
            end
         end
     
         point1 = comp_centerpt
         point2 = point1.offset projected_vector
      
         #perform a raytest to determine whether the line intersects the edge or something else
         temp_vector = point1.vector_to point2
         temp_ray = point1, temp_vector
         temp_item = mod.raytest(temp_ray,false) #explicit declaration of false flag is necessary when layer visibility is utilized

#         if segment.bounds.contains?(temp_item[0])
         if temp_item[0].on_line? segment.line
            cand_edge_startpts << point1   
            cand_edge_endpts << point2
            tempdist = point1.distance(point2)
            testdist << tempdist
            current_loop_index = loop_counter
         end  
      end # if distance exclusion
   
   if cand_edge_startpts.length >= 1 && (loop_counter - current_loop_index) > 25
      break
   end

   end # hallpath do

   #identify the shortest candidate edge for placement
   if !testdist[0].nil?
      index = testdist.each_with_index.min
      if (cand_edge_startpts[ index[1] ].distance cand_edge_endpts[ index[1] ] ) < 100*12
         edge_startpts << cand_edge_startpts[ index[1] ]
         edge_endpts << cand_edge_endpts[ index[1] ]
      end
   end

end # sel_comp do

#Draw edges
limit = edge_startpts.length
for e in 0 ... limit
   if !edge_endpts[e].nil?
      ent.add_edges edge_startpts[e], edge_endpts[e]
   end    
end

#shift all comps downward
allcomps.each do |e|
   v = Z_AXIS.clone
   if e.definition.name.include?("Profile")
      v.length = -2
   else
      v.length = -1
   end
   e.transform!(v)
end

finish = Time.now

run_time = finish - start
text = "#{edge_startpts.length} edges added" + "\n" +
       "#{run_time} s"
UI.messagebox(text, MB_OK)


mod.commit_operation

	end # .main method
 
end # Connect_components_with_edges module

#----------------menu------------------
if !file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Step 2 - Connect Components With Edges") { Connect_components_with_edges.main }
end
 
file_loaded(__FILE__)
