#Step 3 - Combine Close Vertices
#Version 1.1

require 'sketchup.rb'

module Combine_close_vertices
 
  def Combine_close_vertices.main

mod = Sketchup.active_model # Open model
ent = mod.entities # All entities in model
sel = mod.selection # Current selection
mod.start_operation "Combine Close Vertices"

allcomps = []
ent.each do |e|
   allcomps << e if e.is_a?(Sketchup::ComponentInstance) 
end

#shift all comps upward
allcomps.each do |e|
   v = Z_AXIS.clone
   if e.definition.name.include?("Profile")
      v.length = 2
   else
      v.length = 1
   end
   e.transform!(v)
end

#iterate the combine vertices loop until there are none closer than the pre-determined distance
text = []
loop_iteration = 1
loop do

   hallpath = []
   sel.each do |e|
      hallpath << e if e.is_a?(Sketchup::Edge) && e.faces[0].nil?
   end

   count = 0

   #iterate through each hallpath segment to find segments closer than a pre-determined distance and merge the start and end points to a single point
   hallpath.each do |segment|
      if segment.length <= 36 # && segment.start.edges.length > 1 && segment.end.edges.length > 1
         newpoint = []
         allvertices = []
         allvertices << segment.start
         allvertices << segment.end  
         
         #set segement midpoint as the new point
         newpoint = segment.bounds.center
         
         #check if any of the segement endpoints intersect a node or door component.  If so, set the component centerpoint as the new point
         allvertices.each do |f|
            v = Z_AXIS.clone
            ray = f, v
            result = mod.raytest(ray,false) #explicit declaration of false flag is necessary when layer visibility is utilized 
            if (!result.nil? && result[1][0].is_a?(Sketchup::ComponentInstance) && result[1][0].definition.name.start_with?('Door','Room','Stair') )
               newpoint = result[1][0].bounds.center
               newpoint.z = f.position.z
            end
         end
          
         startpt_vector = segment.start.position.vector_to newpoint
         startpt_trans = Geom::Vector3d.new(startpt_vector)
      
         endpt_vector = segment.end.position.vector_to newpoint    
         endpt_trans = Geom::Vector3d.new(endpt_vector)
      
         start_pt = segment.start
         end_pt = segment.end

         ent.transform_entities(startpt_trans, start_pt)
         ent.transform_entities(endpt_trans, end_pt)
         count = count + 1
      end
   end

text << "Iteration: #{loop_iteration} => Segments Moved: #{count} \n"
loop_iteration = loop_iteration + 1

break if count == 0

### the purpose of the following code is to 'clean up' the zero-length edges created by merging the start and end vertices of short edges into a single point.
### first each constrained vertex is selected, then a temporary edge is placed that is then added to a group, the group exploded, and then each edge deleted.
   hallpath_vertices = []
   hallpath.each do |segment|
     if segment.start.edges.length > 1
        hallpath_vertices << segment.start
     elsif segment.end.edges.length > 1
        hallpath_vertices << segment.end
     end
   end
   hallpath_vertices = hallpath_vertices.uniq!

   temp_edges = []
   group = ent.add_group

   hallpath_vertices.each do |e|
      point1 = e.position
      point2 = point1.clone
      point2.x = point2.x + 1
	  point2.y = point2.y + 1
	  point2.z = point2.z + 1
      group.entities.add_edges(point1, point2)
   end

   temp_edges = group.explode
 
   temp_edges.each do |e| 
      if e.is_a? Sketchup::Edge
         e.erase!
      end
   end

end # loop do

#shift all comps downward
allcomps.each do |e|
   v = Z_AXIS.clone
   if e.definition.name.include?("Profile")
      v.length = -2
   else
      v.length = -1
   end
   e.transform!(v)
end

UI.messagebox(text.join, MB_OK)

mod.commit_operation

	end # .main method
 
end # Combine_close_vertices module

#----------------menu------------------
if !file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Step 3 - Combine Close Vertices") { Combine_close_vertices.main }
end
 
file_loaded(__FILE__)
