#Step 5 - Break Curves that Intersect Nodes
#Version 1.1

#This step must be preceeded by running the Sketchup Plugin Edge Tools -> Simplify Curves with a Max Deviation of "0"
#This step will break any curves created that intersect with a Room, Stair, or Elevator, node in the current selection
#Exercise care in using this plugin in a drawing with layers, as wall geometry that intersects with nodes will be moved to the currently selected layer
#Either use the plugin one layer at a time, or shift the node components and connecting paths to an area of the drawing away from the wall geometry

require 'sketchup.rb'

module Break_curves
 
  def Break_curves.main

mod = Sketchup.active_model # Open model
ent = mod.entities # All entities in model
sel = mod.selection # Current selection
mod.start_operation "Break Curves that Intersect Nodes"

allcomps = []
nodes = []
sel.each do |e|
   allcomps << e if e.is_a?(Sketchup::ComponentInstance) 
end
nodes = allcomps.select {|e| e.definition.name.start_with?('Room','Stair','Elevator')}

temp_edges = []
group = ent.add_group

nodes.each do |e|
   point1 = e.bounds.center
   point2 = point1.clone
   point2.x = point2.x + 1
   point2.y = point2.y + 1
   point2.z = point2.z + 1
   group.entities.add_edges(point1, point2)
end

temp_edges = group.explode
 
temp_edges.each do |e| 
   if e.is_a? Sketchup::Edge
      e.erase!
   end
end

d = Sketchup::Set.new

sel.each do |e|
   if e.is_a?(Sketchup::Edge) && e.curve && !e.curve.is_a?(Sketchup::ArcCurve)
      d.insert e.curve
   end
end

count = 0
d.each do |e|
  numedges = e.count_edges
  if numedges == 1
     e.first_edge.explode_curve
     count = count + 1
  end
end

text = "#{d.size} curve(s) identified" + "\n" +
       "#{count} single edge curve(s) removed"
UI.messagebox(text, MB_OK)

mod.commit_operation

	end # .main method
 
end # Break_curves module

#----------------menu------------------
if !file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Step 5 - Break Curves that Intersect Nodes") { Break_curves.main }
end
 
file_loaded(__FILE__)
