#Step 6 - Remove Orphan Edges
#Version 1.1

require 'sketchup.rb'

module Remove_orphan_edges
 
   def Remove_orphan_edges.main

mod = Sketchup.active_model # Open model
ent = mod.entities # All entities in model
sel = mod.selection # Current selection
mod.start_operation "Remove Orphan Edges"

hallpath = []
allcomps = []
orphan_edges = []
 
#select all components from all entities in the model
ent.each do |e|
   allcomps << e if e.is_a?(Sketchup::ComponentInstance) 
end

#shift all comps upward
allcomps.each do |e|
   v = Z_AXIS.clone
   if e.definition.name.include?("Profile")
      v.length = 2
   else
      v.length = 1
   end
   e.transform!(v)
end

#select all hallpaths in the current slection
sel.each do |e|
   hallpath << e if e.is_a? Sketchup::Edge
end

hallpath.each do |e|
   allvertices = []
   allvertices << e.start
   allvertices << e.end
   
   allvertices.each do |f|
      if f.edges.size == 1
         v = Z_AXIS.clone
         ray = f, v
         result = mod.raytest(ray,false) #explicit declaration of false flag is necessary when layer visibility is utilized       
         if result.nil? || ( !result.nil? && result[1][0].is_a?(Sketchup::ComponentInstance) && ( result[1][0].definition.name.start_with?("Door") || result[1][0].definition.name.include?("Profile") ) )
            orphan_edges << e
         end
      end
   end
end

curve_count = 0

orphan_edges.each do |e|
   if e.curve
      curve_count += 1
      e.curve.each_edge { |j|
         j.explode_curve
         orphan_edges << j
      }
   end
end

orphan_edges.uniq

mod.active_entities.erase_entities orphan_edges

#shift all comps downward
allcomps.each do |e|
   v = Z_AXIS.clone
   if e.definition.name.include?("Profile")
      v.length = -2
   else
      v.length = -1
   end
   e.transform!(v)
end

if !orphan_edges.nil?
   edges_deleted = orphan_edges.size
else
   edges_deleted = 0
end

text = "#{edges_deleted} orphan edges (#{curve_count} curves) erased"
UI.messagebox(text, MB_OK)

mod.commit_operation

	end # .main method
 
end # Remove_orphan_edges module

#----------------menu------------------
if !file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Step 6 - Remove Orphan Edges") { Remove_orphan_edges.main }
end
 
file_loaded(__FILE__)
