#Step 7 - Export Network
#Version 1.1

#Before running, ensure that all edges with "vertices" are defined as Curves (run Edge Tools - Simplify Curves
#with a deviation of 0 ), so that vertices are defined distinctly from node endpoints.
#Also, if it is desired for each Room component to be a node, all Curves that intersect the room should have
#and endpoint at the Room centerpoint (Break Curves step)
#Additionally, Remove Orphan Edges will ensure that no edges arrive at at "dead end", either in empty space or in a doorway

require 'sketchup.rb'

module Export_network
 
  def Export_network.main

mod = Sketchup.active_model # Open model
ent = mod.entities # All entities in model
sel = mod.selection # Current selection
mod.start_operation "Export Network"

d = Sketchup::Set.new
paths_log = []
vertices_log = []
nodes_log = []
node_names = {}
path_names = {}

sel.each do |e|

   if e.is_a?(Sketchup::Edge) && e.curve
      d.insert e.curve
   end 
  
   if e.is_a?(Sketchup::Edge) && !e.curve
      start_pt = e.vertices.first
      end_pt = e.vertices.last
      nodes_log << start_pt
      nodes_log << end_pt

      path_record = [e, start_pt, end_pt]
      paths_log << path_record
      
      #add path name to path_name hash
      midpt = e.bounds.center
      path_name = "Path_#{midpt.x.to_mm.round(0)}_#{midpt.z.to_mm.round(0)}_#{midpt.y.to_mm.round(0)*-1}"    
      path_name.gsub! '-', 'N'
      path_names[e] = path_name
   end 

end

d.each do |e|
   vertex_count =  e.vertices.length
   for i in 1..(vertex_count - 2)
      location = e.vertices[i].position
      vertex_record = [e, location.x.to_m.to_f.round(6), location.z.to_m.to_f.round(6), location.y.to_m.to_f.round(6)*-1]
      vertices_log << vertex_record
   end
   
   start_pt = e.vertices.first
   end_pt = e.vertices.last
   
   nodes_log << start_pt
   nodes_log << end_pt
   
   path_record = [ e, start_pt, end_pt]
   paths_log << path_record
   
   #add path name to path_name hash
   midpt = Geom::Point3d.linear_combination(0.5,start_pt.position, 0.5,end_pt.position) 
   path_name = "Path_#{midpt.x.to_mm.round(0)}_#{midpt.z.to_mm.round(0)}_#{midpt.y.to_mm.round(0)*-1}"
   path_name.gsub! '-', 'N'
   path_names[e] = path_name
end

nodes_log == nodes_log.uniq!

#add node name to node_name hash
nodes_log.each do |e|
   location = e.position
   node_name = "Node_#{location.x.to_mm.round(0)}_#{location.z.to_mm.round(0)}_#{location.y.to_mm.round(0)*-1}"
   node_name.gsub! '-', 'N'
   node_names[e] = node_name
end

nodes_log.map! {|e| ["NetworkNode",e, e.position.x.to_m.to_f.round(6), e.position.z.to_m.to_f.round(6), e.position.y.to_m.to_f.round(6)*-1] }
paths_log.map! {|e| e.unshift("NetworkPath")}

###special case to remove duplicate edge paths from paths_log, if one or more curve path connects the same 2 nodes
#note that multiple curve paths could remain, this only takes care of edges
#if this behavior is not desired (i.e. all paths should be exported) this method should be removed.
dup_edges = []
paths_log.each do |e|
   paths_log.each do |j|
      if ( e[2]==j[2] && e[3]==j[3] && e[1]!=j[1]  && !e[1].is_a?(Sketchup::Curve) )
         dup_edges << e
      end  
   end
end

if !dup_edges.nil?
   dup_edges = dup_edges.uniq!
end

if !dup_edges.nil?
   dup_edges.each do |e|
      path_names.delete(e[1])
      e[1].erase!
   end
   paths_log = paths_log - dup_edges

   print dup_edges.length, " duplicate edge path(s) deleted"
   puts
end
### end - special case for duplicate edge paths

#duplicate path name handling
dup_path_names = {}
dup_paths =  path_names.group_by { |e| e[1] }

dup_paths.each do |e|
   counter = 1
   if e[1].length > 1
      e[1].each do |j|
         new_name = "#{j[1]}_#{counter}"
         dup_path_names[j[0]] = new_name
         counter += 1
      end
   end
end

path_names.update(dup_path_names)

#special case for RoomNodes, StairNodes, and Elevator Nodes
allcomps = []
rooms = []
stairs = []
elevators = []
ent.each do |e|
   allcomps << e if e.is_a?(Sketchup::ComponentInstance) 
end
rooms = allcomps.select {|e| e.definition.name.start_with?("Room")}
stairs = allcomps.select {|e| e.definition.name.start_with?("Stair")}
elevators = allcomps.select {|e| e.definition.name.start_with?("Elevator")}

room_names = {}
rooms.each do |e|
   node_names.each do |j|
      if e.bounds.center == j[0].position
         alpha_name = e.name
         alpha_name.gsub! /[^A-Za-z0-9_]/, '_'
         room_names[j[0]] = alpha_name
      end
   end
end

stair_names = {}
stairs.each do |e|
   node_names.each do |j|
      if e.bounds.center == j[0].position
         alpha_name = e.name
         alpha_name.gsub! /[^A-Za-z0-9_]/, '_'
         stair_names[j[0]] = alpha_name
      end
   end
end

elevator_names = {}
elevators.each do |e|
   node_names.each do |j|
      if e.bounds.center == j[0].position
         alpha_name = e.name
         alpha_name.gsub! /[^A-Za-z0-9_]/, '_'
         elevator_names[j[0]] = alpha_name
      end
   end
end

node_names.update(room_names)
node_names.update(stair_names)
node_names.update(elevator_names)

#special case for DoorNodes and DoorPaths
doors = allcomps.select {|e| e.definition.name.start_with? ('Door')}

doors.each do |e|
   @dictionary_name = "dynamic_attributes"
            
   doorthickness = e.definition.get_attribute(@dictionary_name,'doorthickness').to_f
   doorwidth = e.definition.get_attribute(@dictionary_name,'doorwidth').to_f 
   
   if e.definition.name.start_with? ('Door_RH')
      doorstart = e.transformation.origin.offset e.transformation.yaxis, (doorthickness / 2 )
   elsif e.definition.name.start_with? ('Door_LH')
      doorstart = e.transformation.origin.offset e.transformation.yaxis, (doorthickness / -2 )
   end
   doorend = doorstart.offset e.transformation.xaxis, doorwidth

   door_start_name = "DoorNode_#{doorstart.x.to_mm.round(0)}_#{doorstart.z.to_mm.round(0)}_#{doorstart.y.to_mm.round(0)*-1}"
   door_start_name.gsub! '-', 'N'
   nodes_log << ["DoorNode",door_start_name, doorstart.x.to_m.to_f.round(6), doorstart.z.to_m.to_f.round(6), doorstart.y.to_m.to_f.round(6)*-1]

   door_end_name = "DoorNode_#{doorend.x.to_mm.round(0)}_#{doorend.z.to_mm.round(0)}_#{doorend.y.to_mm.round(0)*-1}"
   door_end_name.gsub! '-', 'N'
   nodes_log << ["DoorNode",door_end_name, doorend.x.to_m.to_f.round(6), doorend.z.to_m.to_f.round(6), doorend.y.to_m.to_f.round(6)*-1]
   
   midpt = Geom::Point3d.linear_combination(0.5,doorstart, 0.5,doorend) 
   path_name = "DoorPath_#{midpt.x.to_mm.round(0)}_#{midpt.z.to_mm.round(0)}_#{midpt.y.to_mm.round(0)*-1}"
   path_name.gsub! '-', 'N'
   path_record = ["DoorPath", path_name, door_start_name, door_end_name]
   paths_log << path_record
end

#substitute hash name values for nodes and paths
nodes_log.each {|e|
  node_names.each {|j|
     if e[1] == j[0]
        e[1] = j[1]
     end
  }  
}

paths_log.each {|e|
  path_names.each {|j|
     if e[1] == j[0]
        e[1] = j[1]
     end
  }  
  
  node_names.each {|j|
     if e[2] == j[0]
        e[2] = j[1]
     end
     if e[3] == j[0]
        e[3] = j[1]
     end
  }
}

vertices_log.each {|e|
  path_names.each {|j|
     if e[0] == j[0]
        e[0] = j[1]
     end
  }  
}

#update object class name for the special case of RoomNodes, StairNodes, and Elevator Nodes
nodes_log.each do |e|
   if e[1].start_with?("Room")
      e[0] = "RoomNode"
   end
   if e[1].include?("STAIR")
      e[0] = "StairNode"
   end
   if e[1].include?("ELEV")
      e[0] = "ElevatorNode"
   end
end

#update object class name for the special case of StairPaths
paths_log.each do |e|
   if e[2].include?("STAIR") && e[3].include?("STAIR")
      e[0] = "StairPath"
   end
end

#update object class name for the special case of ElevatorPaths
paths_log.each do |e|
   if e[2].include?("ELEV") && e[3].include?("ELEV")
      e[0] = "ElevatorPath"
   end
end

#add floor identifier to nodes_log
floor_table = [ [-14.6304,"B4"],
				[-10.9728,"B3"],
                [-7.3152,"B2"],
                [-3.6576,"B1"],
                [0.0,"1"],
                [3.6576,"2"],
                [7.3152,"3"],
                [10.9728,"4"],
                [14.6304,"5"],
                [18.288,"6"],
                [21.9456,"7"],
                [25.6032,"8"],
                [29.2608,"9"],
                [32.9184,"10"],
                [36.576,"11"],
                [40.2336,"12"],
                [43.8912,"13"],
                [47.5488,"14"],
                [51.2064,"15"] ]

nodes_log.each {|e|
   floor_table.each {|j|
      if e[3] == j[0]
         e[12] = j[1]
      end
   }
}

nodes_log.sort_by! { |e| [ e[0], e[3], e[1] ] } 
paths_log.sort_by! { |e| e[0] } 

#add RoomUID to nodes_log
room_uid = 1
nodes_log.each do |e|
   if e[0] =="RoomNode"
      e[13] = room_uid
      room_uid += 1
   end
end


#print all Objects
filepath = ENV["USERPROFILE"] + '\Desktop\Objects.csv'
File.open( filepath,'w') do |s|
   s.print "Object Class,Object Name,X,Y,Z,Length,Width,Height,EntityType,InitialNumberInSystem,RideOnTransporter,TransporterName,Floor,RoomUID,PropertyC"
   s.puts
   nodes_log.each do |e|
      s.print e.join(",")
      s.puts
   end
end

#print all Links
filepath = ENV["USERPROFILE"] + '\Desktop\Links.csv'
File.open( filepath,'w') do |s|
   s.print "Link Class,Link Name,From Node,To Node,Network,Type,PropertyA,PropertyB,PropertyC"
   s.puts
   paths_log.each do |e|
      s.print e.join(",")
      s.puts
   end
end

#print all Vertices
filepath = ENV["USERPROFILE"] + '\Desktop\Vertices.csv'
File.open( filepath,'w') do |s|
   s.print "Link Name,Vertex X,Vertex Y,Vertex Z"
   s.puts
   vertices_log.each do |e|
      s.print e.join(",")
      s.puts
   end
end

text = "#{nodes_log.length} objects written to file" + "\n" +
       "#{paths_log.length} links written to file" + "\n" +
       "#{vertices_log.length} vertices written to file"
UI.messagebox(text, MB_OK)

mod.commit_operation

	end # .main method
 
end # Export_network module

#----------------menu------------------
if !file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Step 7 - Export Network") { Export_network.main }
end
 
file_loaded(__FILE__)

